package sun.nio.ch;

import java.util.concurrent.TimeUnit;
import java.util.concurrent.ExecutionException;
import java.io.IOException;
import java.util.concurrent.Future;

final class CompletedFuture<V> implements Future<V>
{
    private final V result;
    private final Throwable exc;
    
    private CompletedFuture(V result, Throwable exc) {
        this.result = result;
        this.exc = exc;
    }
    
    static <V> CompletedFuture<V> withResult(V v) {
        return new CompletedFuture(v, null);
    }
    
    static <V> CompletedFuture<V> withFailure(Throwable cause) {
        if (!(cause instanceof IOException) && !(cause instanceof SecurityException)) {
            cause = new IOException(cause);
        }
        return new CompletedFuture(null, cause);
    }
    
    static <V> CompletedFuture<V> withResult(V v, Throwable t) {
        if (t == null) {
            return withResult(v);
        }
        return withFailure(t);
    }
    
    @Override
    public V get() throws ExecutionException {
        if (this.exc != null) {
            throw new ExecutionException(this.exc);
        }
        return this.result;
    }
    
    @Override
    public V get(long n, TimeUnit timeUnit) throws ExecutionException {
        if (timeUnit == null) {
            throw new NullPointerException();
        }
        if (this.exc != null) {
            throw new ExecutionException(this.exc);
        }
        return this.result;
    }
    
    @Override
    public boolean isCancelled() {
        return false;
    }
    
    @Override
    public boolean isDone() {
        return true;
    }
    
    @Override
    public boolean cancel(boolean b) {
        return false;
    }
}
